#include <XnCppWrapper.h>
using namespace xn;

//Some parts of this code i get from: http://groups.google.com/group/openni-dev/browse_thread/thread/82ecc2bb7f785ec8?pli=1

// this macro need (XnStatus) rc and (xn::NodeInfoList) nodes
#define VALIDATE_GENERATOR(type, desc, generator)                \
    {                                                            \
        rc = m_Context.EnumerateExistingNodes(nodes, type);        \
        if (!nodes.IsEmpty())                                    \
        {                                                        \
            (*(nodes.Begin())).GetInstance(generator);            \
        }                                                        \
        else                                                    \
            printf("No %s generator!\n", desc);                    \
    }

#define CHECK_RC(rc, what)                        \
    CHECK_RC_RET_VAR(rc, what, rc)

#define CHECK_RC_B(rc, what)                    \
    CHECK_RC_RET_VAR(rc, what, false)

#define CHECK_RC_RET_VAR(rc, what, var)                            \
    if (rc != XN_STATUS_OK)                                        \
    {                                                            \
        printf("%s failed: %s\n", what, xnGetStatusString(rc));    \
        return var;                                                \
    }

bool initFromRecordFile(Context m_Context, const String& filename)
{
    XnStatus rc = XN_STATUS_OK;
    rc = m_Context.Init();
    CHECK_RC_B(rc, "Init");
    rc = m_Context.OpenFileRecording(filename.c_str());
    CHECK_RC_B(rc, "Open record file");

    rc = setupNodes();
    CHECK_RC_B(rc, "SetupNodes");

    // Make sure OpenNI nodes start generating
    rc = m_Context.StartGeneratingAll();
    CHECK_RC_B(rc, "StartGenerating");

    return true;
}

setupNodes()
{
    XnStatus rc = XN_STATUS_OK;
    // Make sure we have all OpenNI nodes we will be needing for
    this
    xn::NodeInfoList nodes;

    VALIDATE_GENERATOR(XN_NODE_TYPE_DEPTH, "Depth",
                       m_DepthGenerator);
    if(nodes.IsEmpty())
        return 1;

    VALIDATE_GENERATOR(XN_NODE_TYPE_USER, "User", m_UserGenerator);
    if(nodes.IsEmpty())
    {
        rc = m_UserGenerator.Create(m_Context);
        CHECK_RC(rc, "Find user generator");
    }

    return rc;
}
